package service

import (
	"encoding/json"
	"fmt"
	"gitea.pena/SQuiz/common/model"
)

type Props struct {
	QuizSettings *QuizSettings `json:"quizSettings,omitempty"`
	QuizID       string        `json:"quizId"`
	//Preview                 *bool         `json:"preview,omitempty"`
	//ChangeFaviconAndTitle   *bool         `json:"changeFaviconAndTitle,omitempty"`
	//ClassName               *string       `json:"className,omitempty"`
	//DisableGlobalCss        *bool         `json:"disableGlobalCss,omitempty"`
}

type QuizSettings struct {
	Questions         []Question `json:"questions"`
	Settings          Settings   `json:"settings"`
	Cnt               uint64     `json:"cnt"`
	RecentlyCompleted bool       `json:"recentlyCompleted"`
	ShowBadge         bool       `json:"show_badge"`
}

type Question struct {
	BackendID           int     `json:"backendId"`
	ID                  string  `json:"id"`
	QuizID              int     `json:"quizId"`
	Title               string  `json:"title"`
	Description         string  `json:"description"`
	Page                int     `json:"page"`
	Type                *string `json:"type"`
	Expanded            bool    `json:"expanded"`
	OpenedModalSettings bool    `json:"openedModalSettings"`
	Deleted             bool    `json:"deleted"`
	Required            bool    `json:"required"`
	DeleteTimeoutID     int     `json:"deleteTimeoutId"`
	Content             Content `json:"content"`
}

type Content struct {
	ID           string  `json:"id"`
	Hint         Hint    `json:"hint"`
	Rule         Rule    `json:"rule"`
	Back         *string `json:"back"`
	OriginalBack *string `json:"originalBack"`
	Autofill     bool    `json:"autofill"`
}

type Hint struct {
	Text  string `json:"text"`
	Video string `json:"video"`
}

type Rule struct {
	Children []interface{} `json:"children"`
	Default  string        `json:"default"`
	Main     []interface{} `json:"main"`
	ParentID string        `json:"parentId"`
}

type Settings struct {
	FP       bool   `json:"fp"`
	REP      bool   `json:"rep"`
	Name     string `json:"name"`
	CFG      Config `json:"cfg"`
	LIM      int    `json:"lim"`
	DUE      int    `json:"due"`
	Delay    int    `json:"delay"`
	Pausable bool   `json:"pausable"`
	Status   string `json:"status"` // "start" | "stop" | "ai"
}

type Config struct {
	IsUnSc              *bool       `json:"isUnSc,omitempty"`
	Spec                *bool       `json:"spec,omitempty"`
	Type                *string     `json:"type"` // "quiz" | "form" | null
	NoStartPage         bool        `json:"noStartPage"`
	StartpageType       *string     `json:"startpageType"` // "standard" | "expanded" | "centered"
	Score               *bool       `json:"score,omitempty"`
	Results             *bool       `json:"results"`
	HaveRoot            *string     `json:"haveRoot"`
	Theme               string      `json:"theme"`
	Design              bool        `json:"design"`
	ResultInfo          ResultInfo  `json:"resultInfo"`
	Startpage           Startpage   `json:"startpage"`
	FormContact         FormContact `json:"formContact"`
	Info                Info        `json:"info"`
	Meta                string      `json:"meta"`
	Antifraud           *bool       `json:"antifraud,omitempty"`
	Showfc              *bool       `json:"showfc,omitempty"`
	YandexMetricsNumber *int        `json:"yandexMetricsNumber,omitempty"`
	VkMetricsNumber     *int        `json:"vkMetricsNumber,omitempty"`
}

type ResultInfo struct {
	When           string `json:"when"` // "email" | ""
	Share          bool   `json:"share"`
	Replay         bool   `json:"replay"`
	Theme          string `json:"theme"`
	Reply          string `json:"reply"`
	Replname       string `json:"replname"`
	ShowResultForm string `json:"showResultForm"` // "before" | "after"
}

type Startpage struct {
	Description  string     `json:"description"`
	Button       string     `json:"button"`
	Position     string     `json:"position"` // "left" | "right" | "center"
	FavIcon      *string    `json:"favIcon"`
	Logo         *string    `json:"logo"`
	OriginalLogo *string    `json:"originalLogo"`
	Background   Background `json:"background"`
}

type Background struct {
	Type            *string `json:"type"` // null | "image" | "video"
	Desktop         *string `json:"desktop"`
	OriginalDesktop *string `json:"originalDesktop"`
	Mobile          *string `json:"mobile"`
	OriginalMobile  *string `json:"originalMobile"`
	Video           *string `json:"video"`
	Cycle           bool    `json:"cycle"`
}

type FormContact struct {
	Title  string                 `json:"title"`
	Desc   string                 `json:"desc"`
	Fields map[string]interface{} `json:"fields"`
	Button string                 `json:"button"`
}

type Info struct {
	PhoneNumber string  `json:"phonenumber"`
	Clickable   bool    `json:"clickable"`
	OrgName     string  `json:"orgname"`
	Site        string  `json:"site"`
	Law         *string `json:"law,omitempty"`
}

func dao2dtoProps(quiz model.Quiz, questions []model.Question, showBadge bool, cnt uint64) Props {
	var cfg Config
	err := json.Unmarshal([]byte(quiz.Config), &cfg)
	if err != nil {
		fmt.Printf("failed to unmarshal quiz.Config: %v\n", err)
		cfg = Config{}
	}

	return Props{
		QuizID: quiz.Qid,
		QuizSettings: &QuizSettings{
			Settings: Settings{
				FP:       quiz.Fingerprinting,
				REP:      quiz.Repeatable,
				Name:     quiz.Name,
				CFG:      cfg,
				LIM:      int(quiz.Limit),
				DUE:      int(quiz.DueTo),
				Delay:    int(quiz.TimeOfPassing),
				Pausable: quiz.Pausable,
				Status:   quiz.Status,
			},
			Cnt:       cnt,
			ShowBadge: showBadge,
			Questions: dao2dtoQuestions(int(quiz.Id), questions),
		},
	}
}

func dao2dtoQuestions(quizId int, questions []model.Question) []Question {
	var res []Question
	for _, q := range questions {
		var content Content
		err := json.Unmarshal([]byte(q.Content), &content)
		if err != nil {
			fmt.Printf("failed to unmarshal question.Content for ID %d: %v\n", q.Id, err)
			content = Content{}
		}

		t := q.Type
		res = append(res, Question{
			BackendID: int(q.Id),
			//ID:                  "",
			QuizID:      quizId,
			Title:       q.Title,
			Description: q.Description,
			Page:        q.Page,
			Type:        &t,
			Deleted:     q.Deleted,
			Required:    q.Required,
			Content:     content,
		})
	}
	return res
}
